describe('EditAction', function() {

    it('should save undo and redo actions', function() {
        let flag = false;
        let e = new EditAction("0", function() {
            flag = false;
        }, function() {
            flag = true;
        }, null);
        e.activate();
        e.undo();
        expect(flag).toBeTruthy();
        e.redo();
        expect(flag).toBeFalsy();
    });


});

describe('ActionHistory', function() {

    it('should save added actions', function() {
        let history = new ActionHistory();
        history.addAction("0", function() {
            flag = false;
        }, function() {
            flag = true;
        });

        expect(history.getActions().length).toBe(1);
        expect(history.getIndex()).toBe(0);

        history.addAction("1", function() {
            flag = false;
        }, function() {
            flag = true;
        });

        expect(history.getActions().length).toBe(2);
        expect(history.getIndex()).toBe(1);

    });

    it('should undo last action', function() {
        let history = new ActionHistory();
        let flag1 = true;
        let flag2 = true;
        act1 = history.addAction("0", function() {
            flag1 = true;
        }, function() {
            flag1 = false;
        });

        act2 = history.addAction("1", function() {
            flag2 = true;
        }, function() {
            flag2 = false;
        });

        act2.activate();

        let success = history.undo();
        expect(success).toBeTruthy();
        expect(history.getActions().length).toBe(2);
        expect(history.getIndex()).toBe(0);
        expect(flag2).toBeFalsy();
        expect(flag1).toBeTruthy();

    });

    it('should pass action to undo', function() {
        let history = new ActionHistory();
        let flag1 = true;
        act1 = history.addAction("0", function(action) {
            flag1 = true;
        }, function(action) {
            flag1 = false;
            expect(action).toBe(act1);
        });

        act1.activate();

        let success = history.undo();
        expect(success).toBeTruthy();
        expect(history.getActions().length).toBe(1);
        expect(history.getIndex()).toBe(-1);
        expect(flag1).toBeFalsy();

    });

    it('should fail to undo deactivated action', function() {
        let history = new ActionHistory();
        let flag1 = true;
        act1 = history.addAction("0", function() {
            flag1 = true;
        }, function() {
            flag1 = false;
        });

        let success = history.undo();
        expect(success).toBeFalsy();
        expect(history.getActions().length).toBe(1);
        expect(history.getIndex()).toBe(0);
        expect(flag1).toBeTruthy();

    });

    it('should redo after undo', function() {
        let history = new ActionHistory();
        let flag1 = true;
        let flag2 = true;
        act1 = history.addAction("0", function() {
            flag1 = true;
        }, function() {
            flag1 = false;
        });

        act2 = history.addAction("1", function() {
            flag2 = true;
        }, function() {
            flag2 = false;
        });

        act2.activate();

        let success = history.undo();
        expect(success).toBeTruthy();
        expect(history.getActions().length).toBe(2);
        expect(history.getIndex()).toBe(0);
        expect(flag2).toBeFalsy();
        expect(flag1).toBeTruthy();
        act2.activate();

        success = history.redo();
        expect(success).toBeTruthy();
        expect(history.getActions().length).toBe(2);
        expect(history.getIndex()).toBe(1);
        expect(flag2).toBeTruthy();
        expect(flag1).toBeTruthy();

    });

    it('should remove action', function() {
        let history = new ActionHistory();
        let flag1 = true;
        act1 = history.addAction("0", function() {
            flag1 = true;
        }, function() {
            flag1 = false;
        });

        act1.activate();
        let success = history.undo();
        expect(success).toBeTruthy();
        expect(history.getActions().length).toBe(1);
        expect(history.getIndex()).toBe(-1);
        expect(flag1).toBeFalsy();

        act1.activate();
        act1.remove();
        expect(history.getActions().length).toBe(0);
        expect(history.getIndex()).toBe(-1);
    });



});